import json
from datetime import datetime, timezone
import re

import pytest
import responses

from ixp_tracker.conf import DATA_ARCHIVE_URL
from ixp_tracker.importers import import_data
from ixp_tracker.management.commands.ixp_tracker_import import DefaultAdditionalDataSources
from ixp_tracker.models import ASN, IXP, IXPMember

from .fixtures import PeeringASNFactory, PeeringIXFactory, PeeringNetIXLANFactory

pytestmark = pytest.mark.django_db
additional_data = DefaultAdditionalDataSources()


def test_with_no_data_returned_does_nothing():
    backfill_date = datetime(year=2024, month=1, day=1).replace(tzinfo=timezone.utc)
    with responses.RequestsMock() as rsps:
        rsps.get(
            url=DATA_ARCHIVE_URL.format(year=backfill_date.year, month=backfill_date.month, day=backfill_date.day),
            body=json.dumps({"ix": {"data": []}, "net": {"data": []}, "netixlan": {"data": []}}),
        )
        import_data(additional_data, False, backfill_date)

    ixps = IXP.objects.all()
    assert len(ixps) == 0

    asns = ASN.objects.all()
    assert len(asns) == 0

    members = IXPMember.objects.all()
    assert len(members) == 0


def test_handles_malformed_archives():
    backfill_date = datetime(year=2024, month=1, day=1).replace(tzinfo=timezone.utc)
    with responses.RequestsMock() as rsps:
        rsps.get(
            url=DATA_ARCHIVE_URL.format(year=backfill_date.year, month=backfill_date.month, day=backfill_date.day),
            body=json.dumps({}),
        )
        import_data(additional_data, False, backfill_date)

    ixps = IXP.objects.all()
    assert len(ixps) == 0

    asns = ASN.objects.all()
    assert len(asns) == 0

    members = IXPMember.objects.all()
    assert len(members) == 0


def test_handles_single_quoted_json():
    backfill_date = datetime(year=2024, month=1, day=1).replace(tzinfo=timezone.utc)
    with responses.RequestsMock() as rsps:
        rsps.get(
            url=DATA_ARCHIVE_URL.format(year=backfill_date.year, month=backfill_date.month, day=backfill_date.day),
            body="{'ix': {'data': []}, 'net': {'data': []}, 'netixlan': {'data': []}}",
        )
        import_data(additional_data, False, backfill_date)

    ixps = IXP.objects.all()
    assert len(ixps) == 0

    asns = ASN.objects.all()
    assert len(asns) == 0

    members = IXPMember.objects.all()
    assert len(members) == 0


def test_queries_for_every_day_of_month():
    backfill_date = datetime(year=2024, month=1, day=1)
    with responses.RequestsMock() as rsps:
        data_url = DATA_ARCHIVE_URL.format(year=backfill_date.year, month=backfill_date.month, day=backfill_date.day)
        data_url = data_url.replace("01.json", r'[0-9]{2}\.json')
        rsps.get(
            url=re.compile(data_url),
            status=404,
        )
        import_data(additional_data, False, backfill_date)

    ixps = IXP.objects.all()
    assert len(ixps) == 0

    asns = ASN.objects.all()
    assert len(asns) == 0

    members = IXPMember.objects.all()
    assert len(members) == 0


def test_adds_all_data():
    backfill_date = datetime(year=2024, month=1, day=1).replace(tzinfo=timezone.utc)
    asn_data = PeeringASNFactory()
    ix_data = PeeringIXFactory()
    with responses.RequestsMock() as rsps:
        rsps.get(
            url=DATA_ARCHIVE_URL.format(year=backfill_date.year, month=backfill_date.month, day=backfill_date.day),
            body=json.dumps(
                {
                    "ix": {"data": [ix_data]},
                    "net": {"data": [asn_data]},
                    "netixlan": {"data": [PeeringNetIXLANFactory(asn=asn_data["asn"], ix_id=ix_data["id"])]}
                }
            ),
        )
        import_data(additional_data, False, backfill_date)

    ixps = IXP.objects.all()
    assert len(ixps) == 1

    asns = ASN.objects.all()
    assert len(asns) == 1

    members = IXPMember.objects.all()
    assert len(members) == 1



# Generated by Django 5.2.5 on 2025-09-03 10:36

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('ixp_tracker', '0017_ixp_org_id'),
    ]

    operations = [
        migrations.AddField(
            model_name='ixp',
            name='anchor_host',
            field=models.BooleanField(default=False),
        ),
    ]



from datetime import datetime, timedelta, timezone

import pytest

from ixp_tracker import importers
from ixp_tracker.importers import ASNGeoLookup, dedupe_member_data
from ixp_tracker.models import IXPMember, IXPMembershipRecord
from tests.fixtures import ASNFactory, IXPFactory, PeeringNetIXLANFactory, create_member_fixture

pytestmark = pytest.mark.django_db

date_now = datetime.now(timezone.utc)

class TestLookup(ASNGeoLookup):
    __test__ = False

    def __init__(self, default_status: str = "assigned"):
        self.default_status = default_status

    def get_iso2_country(self, asn: int, as_at: datetime) -> str:
        pass

    def get_status(self, asn: int, as_at: datetime) -> str:
        assert as_at <= datetime.now(timezone.utc)
        assert asn > 0
        return self.default_status


def test_with_no_data_does_nothing():
    processor = importers.process_member_data(date_now, TestLookup())
    processor([])

    members = IXPMember.objects.all()
    assert len(members) == 0


def test_adds_new_member():
    ixp = IXPFactory()
    asn = ASNFactory()
    member_import = PeeringNetIXLANFactory(asn=asn.number, ix_id=ixp.peeringdb_id)

    processor = importers.process_member_data(date_now, TestLookup())
    processor([member_import])

    members = IXPMember.objects.all()
    assert len(members) == 1
    current_membership = IXPMembershipRecord.objects.filter(member=members.first())
    assert len(current_membership) == 1


def test_does_nothing_if_no_asn_found():
    ixp = IXPFactory()
    member_import = PeeringNetIXLANFactory(ix_id=ixp.peeringdb_id)

    processor = importers.process_member_data(date_now, TestLookup())
    processor([member_import])

    members = IXPMember.objects.all()
    assert len(members) == 0


def test_does_nothing_if_no_ixp_found():
    asn = ASNFactory()
    member_import = PeeringNetIXLANFactory(asn=asn.number)

    processor = importers.process_member_data(date_now, TestLookup())
    processor([member_import])

    members = IXPMember.objects.all()
    assert len(members) == 0


def test_updates_existing_member():
    ixp = IXPFactory()
    member = create_member_fixture(ixp)
    member_import = PeeringNetIXLANFactory(asn=member.asn.number, ix_id=ixp.peeringdb_id)

    processor = importers.process_member_data(date_now, TestLookup())
    processor([member_import])

    members = IXPMember.objects.all()
    assert len(members) == 1
    updated = members.first()
    assert updated.last_active > member.last_active


def test_updates_membership_for_existing_member():
    ixp = IXPFactory()
    member = create_member_fixture(ixp, membership_properties={"speed": 500, "is_rs_peer": False})
    member_import = PeeringNetIXLANFactory(asn=member.asn.number, ix_id=ixp.peeringdb_id, speed=10000, is_rs_peer=True)

    processor = importers.process_member_data(date_now, TestLookup())
    processor([member_import])

    membership = IXPMembershipRecord.objects.filter(member=member)
    assert len(membership) == 1
    current_membership = membership.first()
    assert current_membership.is_rs_peer
    assert current_membership.speed == 10000


def test_adds_new_membership_for_existing_member_marked_as_left():
    ixp = IXPFactory()
    member = create_member_fixture(ixp, membership_properties={"start_date": datetime(year=2018, month=1, day=3), "end_date": datetime(year=2018, month=7, day=13, tzinfo=timezone.utc)})
    member_import = PeeringNetIXLANFactory(asn=member.asn.number, ix_id=ixp.peeringdb_id)

    processor = importers.process_member_data(date_now, TestLookup())
    processor([member_import])

    members = IXPMember.objects.all()
    assert len(members) == 1
    current_membership = IXPMembershipRecord.objects.filter(member=member).order_by("-start_date")
    assert len(current_membership) == 2
    assert current_membership.first().end_date is None


def test_extends_membership_for_member_marked_as_left_if_created_before_date_left():
    ixp = IXPFactory()
    member = create_member_fixture(ixp, membership_properties={"start_date": datetime(year=2018, month=1, day=3), "end_date": datetime(year=2018, month=7, day=13, tzinfo=timezone.utc)})
    member_data_with_created_date_before_date_left = PeeringNetIXLANFactory(asn=member.asn.number, ix_id=ixp.peeringdb_id, created_date=datetime(year=2018, month=6, day=18, tzinfo=timezone.utc))

    processor = importers.process_member_data(date_now, TestLookup())
    processor([member_data_with_created_date_before_date_left])

    members = IXPMember.objects.all()
    assert len(members) == 1
    current_membership = IXPMembershipRecord.objects.filter(member=member).order_by("-start_date")
    assert len(current_membership) == 1
    assert current_membership.first().end_date is None


def test_marks_member_as_left_that_is_no_longer_active():
    first_day_of_month = datetime.now(timezone.utc).replace(day=1)
    last_day_of_last_month = (first_day_of_month - timedelta(days=1))
    date_more_than_month_ago = last_day_of_last_month - timedelta(days=17)

    ixp = IXPFactory()
    member = create_member_fixture(ixp, membership_properties={"start_date": date_more_than_month_ago}, member_properties={"last_active": date_more_than_month_ago})

    current_membership = IXPMembershipRecord.objects.filter(member=member)
    assert current_membership.first().end_date is None

    processor = importers.process_member_data(date_now, TestLookup())
    processor([])

    current_membership = IXPMembershipRecord.objects.filter(member=member)
    assert current_membership.first().end_date.strftime("%Y-%m-%d") == last_day_of_last_month.strftime("%Y-%m-%d")


def test_does_not_mark_member_as_left_if_asn_is_registered_in_country_zz_and_is_assigned():
    asn = ASNFactory(registration_country_code="ZZ")
    ixp = IXPFactory()
    member = create_member_fixture(ixp, asn, member_properties={"last_active": datetime.now(timezone.utc)})

    processor = importers.process_member_data(date_now, TestLookup(default_status="assigned"))
    processor([])

    current_membership = IXPMembershipRecord.objects.filter(member=member)
    assert current_membership.first().end_date is None


def test_marks_member_as_left_if_asn_is_registered_in_country_zz_and_is_not_assigned():
    first_day_of_month = datetime.now(timezone.utc).replace(day=1)
    last_day_of_last_month = (first_day_of_month - timedelta(days=1))

    asn = ASNFactory(registration_country_code="ZZ")
    ixp = IXPFactory()
    member = create_member_fixture(ixp, asn, member_properties={"last_active": datetime.now(timezone.utc)})

    processor = importers.process_member_data(date_now, TestLookup("available"))
    processor([])

    current_membership = IXPMembershipRecord.objects.filter(member=member)
    assert current_membership.first().end_date.strftime("%Y-%m-%d") == last_day_of_last_month.strftime("%Y-%m-%d")


def test_does_not_mark_as_left_before_joining_date():
    first_day_of_month = datetime.now(timezone.utc).replace(day=1)

    asn = ASNFactory(registration_country_code="ZZ")
    ixp = IXPFactory()
    member = create_member_fixture(ixp, asn, member_properties={"last_active": datetime.now(timezone.utc)}, membership_properties={"start_date": first_day_of_month})

    processor = importers.process_member_data(date_now, TestLookup("available"))
    processor([])

    current_membership = IXPMembershipRecord.objects.filter(member=member)
    assert current_membership.first().end_date.strftime("%Y-%m-%d") == first_day_of_month.strftime("%Y-%m-%d")


def test_ensure_multiple_member_entries_does_not_trigger_multiple_new_memberships():
    ixp = IXPFactory()
    member = create_member_fixture(ixp, membership_properties={"start_date": datetime(year=2023, month=1, day=13, tzinfo=timezone.utc), "end_date": datetime(year=2023, month=7, day=13, tzinfo=timezone.utc)})

    date_after_date_left = datetime(2023, 9, 24, tzinfo=timezone.utc)
    member_data_with_created_date_after_date_left = PeeringNetIXLANFactory(created_date=date_after_date_left, ix_id=ixp.peeringdb_id, asn=member.asn.number)

    processor = importers.process_member_data(date_now, TestLookup())
    processor([member_data_with_created_date_after_date_left, member_data_with_created_date_after_date_left])

    memberships = IXPMembershipRecord.objects.filter(member=member)
    assert len(memberships) == 2


def test_do_not_add_new_membership_for_same_created_date():
    ixp = IXPFactory()
    created_date = datetime(year=2023, month=1, day=13, tzinfo=timezone.utc)
    member = create_member_fixture(ixp, membership_properties={"start_date": created_date, "end_date": datetime(year=2023, month=7, day=13, tzinfo=timezone.utc)})
    # As we always create a new membership record if the most recent one has ended, for multiple ASN-IX combos this
    # could result in multiple new memberships being created
    member_import = PeeringNetIXLANFactory(created_date=created_date, ix_id=ixp.peeringdb_id, asn=member.asn.number)

    processor = importers.process_member_data(date_now, TestLookup())
    processor([member_import])

    memberships = IXPMembershipRecord.objects.filter(member=member)
    assert len(memberships) == 1


def test_deduplicates_member_data_before_processing():
    member_import = PeeringNetIXLANFactory()
    duplicate_one = PeeringNetIXLANFactory(ix_id=member_import["ix_id"], asn=member_import["asn"])
    duplicate_two = PeeringNetIXLANFactory(ix_id=member_import["ix_id"], asn=member_import["asn"])

    deduplicated_data = dedupe_member_data([member_import, duplicate_one, duplicate_two])

    assert len(deduplicated_data) == 1


def test_set_rs_peer_to_true_if_any_member_is_set_to_true():
    member_import = PeeringNetIXLANFactory(is_rs_peer=False)
    duplicate_one = PeeringNetIXLANFactory(ix_id=member_import["ix_id"], asn=member_import["asn"], is_rs_peer=True)
    duplicate_two = PeeringNetIXLANFactory(ix_id=member_import["ix_id"], asn=member_import["asn"], is_rs_peer=False)

    deduplicated_data = dedupe_member_data([member_import, duplicate_one, duplicate_two])

    deduplicated_member = deduplicated_data[0]
    assert deduplicated_member["is_rs_peer"]


def test_speed_for_deduped_members_is_sum_of_all_speeds():
    member_import = PeeringNetIXLANFactory(speed=500)
    duplicate_one = PeeringNetIXLANFactory(ix_id=member_import["ix_id"], asn=member_import["asn"], speed=1000)
    duplicate_two = PeeringNetIXLANFactory(ix_id=member_import["ix_id"], asn=member_import["asn"], speed=3000)

    deduplicated_data = dedupe_member_data([member_import, duplicate_one, duplicate_two])

    deduplicated_member = deduplicated_data[0]
    assert deduplicated_member["speed"] == 4500



import json
from datetime import datetime, timezone

import pytest
import responses

from django_test_app.settings import IXP_TRACKER_PEERING_DB_URL
from ixp_tracker import importers
from ixp_tracker.models import ASN

from .fixtures import ASNFactory, PeeringASNFactory

pytestmark = pytest.mark.django_db


class TestLookup:

    def get_iso2_country(self, asn: int, as_at: datetime) -> str:
        assert as_at <= datetime.now(timezone.utc)
        assert asn > 0
        return "AU"

    def get_status(self, asn: int, as_at: datetime) -> str:
        pass


def test_with_empty_response_does_nothing():
    processor = importers.process_asn_data(TestLookup())
    processor([])

    asns = ASN.objects.all()
    assert len(asns) == 0


def test_with_no_existing_data_gets_all_data():
    with responses.RequestsMock() as rsps:
        rsps.get(
            url=IXP_TRACKER_PEERING_DB_URL + "/net?limit=200&skip=0",
            body=""
        )
        importers.import_asns(TestLookup(), False)

        asns = ASN.objects.all()
        assert len(asns) == 0


def test_imports_new_asn():
    processor = importers.process_asn_data(TestLookup())
    processor([PeeringASNFactory()])

    asns = ASN.objects.all()
    assert len(asns) == 1


def test_updates_existing_data():
    updated_asn_data = PeeringASNFactory()
    ASNFactory(
        number=updated_asn_data["asn"],
        peeringdb_id=updated_asn_data["id"],
        last_updated=datetime(2024, 5, 1, tzinfo=timezone.utc)
    )
    with responses.RequestsMock() as rsps:
        rsps.get(
            url=IXP_TRACKER_PEERING_DB_URL + "/net?updated__gte=2024-05-01&limit=200&skip=0",
            body=json.dumps({"data": [updated_asn_data]}),
        )
        rsps.get(
            url=IXP_TRACKER_PEERING_DB_URL + "/net?updated__gte=2024-05-01&limit=200&skip=200",
            body=json.dumps({"data": []}),
        )
        importers.import_asns(TestLookup(), False)

        asns = ASN.objects.all()
        assert len(asns) == 1
        updated = asns.filter(peeringdb_id=updated_asn_data["id"]).first()
        assert updated.name == updated_asn_data["name"]
        assert updated.registration_country_code == "AU"


def test_handles_errors_with_source_data():
    data_with_problems = PeeringASNFactory()
    data_with_problems["updated"] = "abc"
    data_with_problems["asn"] = "foobar"

    processor = importers.process_asn_data(TestLookup())
    processor([data_with_problems])

    asns = ASN.objects.all()
    assert len(asns) == 0



# Generated by Django 5.2.5 on 2025-10-14 15:22

import datetime
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('ixp_tracker', '0018_ixp_anchor_host'),
    ]

    operations = [
        migrations.AddField(
            model_name='statspercountry',
            name='last_generated',
            field=models.DateTimeField(default=datetime.datetime.now(datetime.timezone.utc)),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='statsperixp',
            name='last_generated',
            field=models.DateTimeField(default=datetime.datetime.now(datetime.timezone.utc)),
            preserve_default=False,
        ),
    ]



# Generated by Django 5.2.5 on 2025-10-31 11:42

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('ixp_tracker', '0019_statspercountry_last_generated_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='ixp',
            name='physical_locations',
            field=models.IntegerField(default=0),
        ),
    ]



# Generated by Django 5.2.5 on 2025-08-27 12:06

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('ixp_tracker', '0015_statsperixp_monthly_members_change_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='statspercountry',
            name='routed_asns_ixp_member_customers_rate',
            field=models.FloatField(default=0),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='statsperixp',
            name='local_routed_asns_members_customers_rate',
            field=models.FloatField(default=0),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='statspercountry',
            name='routed_asn_count',
            field=models.IntegerField(default=0),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='statspercountry',
            name='routed_asns_ixp_member_rate',
            field=models.FloatField(default=0),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='asn',
            name='peering_policy',
            field=models.CharField(choices=[('open', 'Open'), ('selective', 'Selective'), ('restrictive', 'Restrictive'), ('no', 'No'), ('unknown', 'Unknown')], default='unknown', max_length=50),
        ),
        migrations.AlterField(
            model_name='asn',
            name='network_type',
            field=models.CharField(choices=[('nsp', 'NSP'), ('content', 'Content'), ('cable-dsl-isp', 'Cable/DSL/ISP'), ('enterprise', 'Enterprise'), ('education-research', 'Educational/Research'), ('non-profit', 'Non-Profit'), ('route-server', 'Route Server'), ('network-services', 'Network Services'), ('route-collector', 'Route Collector'), ('government', 'Government'), ('not-disclosed', 'Not Disclosed'), ('other', 'Other'), ('unknown', 'Unknown')], default='unknown', max_length=200),
        ),
    ]



from datetime import datetime, timedelta, timezone

import pytest

from ixp_tracker.models import StatsPerCountry
from ixp_tracker.stats import generate_stats
from tests.fixtures import ASNFactory, IXPFactory, MockLookup, StatsPerCountryFactory, create_member_fixture

pytestmark = pytest.mark.django_db


def test_with_no_data_generates_no_stats():
    generate_stats(MockLookup())

    stats = StatsPerCountry.objects.all()
    assert len(stats) == 249
    first_stat = stats.first()
    assert first_stat.member_count == 0


def test_generates_stats():
    ixp_one = IXPFactory()
    create_member_fixture(ixp_one)
    member_one_multiple_ixps = create_member_fixture(ixp_one)
    member_two_multiple_ixps = create_member_fixture(ixp_one)
    ixp_two = IXPFactory(country_code=ixp_one.country_code)
    create_member_fixture(ixp_two)
    create_member_fixture(ixp_two, asn=member_one_multiple_ixps.asn)
    create_member_fixture(ixp_two, asn=member_two_multiple_ixps.asn)
    non_member_asn = ASNFactory()
    customer_asn = ASNFactory()

    asns_in_country = [member_one_multiple_ixps.asn.number, member_two_multiple_ixps.asn.number, non_member_asn.number]
    routed_asns_in_country = [member_one_multiple_ixps.asn.number, non_member_asn.number, customer_asn, ASNFactory().number]
    generate_stats(MockLookup(asns=asns_in_country, routed_asns=routed_asns_in_country, customer_asns=[customer_asn]))

    stats = StatsPerCountry.objects.filter(country_code=ixp_one.country_code).first()
    assert stats.ixp_count == 2
    assert stats.asn_count == 3
    assert stats.routed_asn_count == 4
    assert stats.member_count == 4
    assert stats.asns_ixp_member_rate == pytest.approx(0.666, abs=0.001)
    assert stats.routed_asns_ixp_member_rate == 0.25
    assert stats.routed_asns_ixp_member_customers_rate == 0.5


def test_generates_ixp_counts():
    stats_date = datetime.now(timezone.utc)
    one_month_before = (stats_date - timedelta(days=1)).replace(day=1)
    one_month_after = (stats_date + timedelta(days=35)).replace(day=1)
    # active_status only stores the current status whereas the stats need to generate historically
    # so the stats use the historical member count rather than active_status
    # currently_active with three members
    active = IXPFactory(active_status=True)
    create_member_fixture(active, membership_properties={"start_date": one_month_before, "end_date": one_month_after}, quantity=3)
    # member active in the past
    member_in_past = IXPFactory(active_status=True, country_code=active.country_code)
    create_member_fixture(member_in_past, membership_properties={"start_date": one_month_before, "end_date": one_month_before})
    # member not yet active (as we are generating historical stats there could be members in the future)
    member_in_future = IXPFactory(active_status=True, country_code=active.country_code)
    create_member_fixture(member_in_future, membership_properties={"start_date": one_month_after})
    # currently_active but only two members
    not_enough_members = IXPFactory(active_status=True, country_code=active.country_code)
    create_member_fixture(not_enough_members, membership_properties={"start_date": one_month_before, "end_date": one_month_after}, quantity=2)

    generate_stats(MockLookup(), stats_date)

    stats = StatsPerCountry.objects.filter(country_code=active.country_code).first()
    assert stats.ixp_count == 1
    assert stats.member_count == 3


def test_handles_invalid_country():
    IXPFactory(country_code="XK")

    generate_stats(MockLookup())

    country_stats = StatsPerCountry.objects.filter(country_code="XK").first()
    assert country_stats is None


def test_updates_existing_stats_entry():
    date_now = datetime.now(timezone.utc)
    # Ensure stats_date and last_generated are always in the past so we can verify the updated last_generated
    stats_date = (date_now.replace(day=1) - timedelta(days=1)).replace(day=1)
    last_generated = stats_date + timedelta(days=1)
    existing = StatsPerCountryFactory(stats_date=stats_date, last_generated=last_generated)

    generate_stats(MockLookup(), stats_date)

    all_stats_for_country = StatsPerCountry.objects.filter(country_code=existing.country_code)
    assert all_stats_for_country.count() == 1
    country_stats = all_stats_for_country.first()
    assert country_stats.last_generated > existing.last_generated
    assert country_stats.ixp_count == 0
    assert country_stats.asn_count == 0
    assert country_stats.member_count == 0



# Generated by Django 5.0.8 on 2025-08-27 09:05

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('ixp_tracker', '0013_ixp_manrs_participant'),
    ]

    operations = [
        migrations.AddField(
            model_name='statsperixp',
            name='members_joined_last_12_months',
            field=models.IntegerField(default=0),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='statsperixp',
            name='members_left_last_12_months',
            field=models.IntegerField(default=0),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='statsperixp',
            name='local_routed_asns_members_rate',
            field=models.FloatField(default=0),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='statsperixp',
            name='rs_peering_rate',
            field=models.FloatField(default=0),
            preserve_default=False,
        ),
    ]



import importlib
import logging
from datetime import datetime
from typing import Protocol

logger = logging.getLogger("ixp_tracker")


class ASNGeoLookup(Protocol):

    def get_iso2_country(self, asn: int, as_at: datetime) -> str:
        pass

    def get_status(self, asn: int, as_at: datetime) -> str:
        pass

    def get_asns_for_country(self, country: str, as_at: datetime) -> list[int]:
        pass

    def get_routed_asns_for_country(self, country: str, as_at: datetime) -> list[int]:
        pass


class ASNCustomerLookup(Protocol):

    def get_customer_asns(self, asns: list[int], as_at: datetime) -> list[int]:
        pass


class MANRSParticipantsLookup(Protocol):

    def get_manrs_participants(self, as_at: datetime) -> list[int]:
        pass


class AtlasAnchorHostLookup(Protocol):

    def get_atlas_anchor_hosts(self, as_at: datetime) -> list[int]:
        pass


class AdditionalDataSources(ASNGeoLookup, ASNCustomerLookup, MANRSParticipantsLookup, AtlasAnchorHostLookup):
    pass


class DefaultAdditionalDataSources(ASNGeoLookup, ASNCustomerLookup, MANRSParticipantsLookup, AtlasAnchorHostLookup):

    def get_iso2_country(self, asn: int, as_at: datetime) -> str:
        return "ZZ"

    def get_status(self, asn: int, as_at: datetime) -> str:
        return "assigned"

    def get_asns_for_country(self, country: str, as_at: datetime) -> list[int]:
        return []

    def get_customer_asns(self, asns: list[int], as_at: datetime) -> list[int]:
        return []

    def get_manrs_participants(self, as_at: datetime) -> list[int]:
        return []
    
    def get_atlas_anchor_hosts(self, as_at: datetime) -> list[int]:
        return []



def load_lookup(lookup_name):
    if lookup_name is not None:
        lookup_parts = lookup_name.split(".")
        factory_name = lookup_parts.pop()
        module_name = ".".join(lookup_parts)
        logger.debug("Trying to load geo lookup", extra={"module_name": module_name, "factory": factory_name})
        if module_name and factory_name:
            imported_module = importlib.import_module(module_name)
            factory = getattr(imported_module, factory_name)
            return factory()
    return None
